/* This is a generated test, do not edit
 * Generated by gen_extensions_defined.py
 *
 * [config]
 * expect_result: pass
 * glsl_version: 1.10
 * require_extensions: !GL_ARB_sample_shading 
 * [end config]
 */

#version 110


#if defined GL_ARB_sample_shading
#    error GL_ARB_sample_shading is defined, but should not be
#endif

float foo() { return 0.0; }
