td {
	padding: 4pt;
}

th {
	padding: 4pt;
	background-color: #c8c838;
}

table {
	border: 0pt;
	border-collapse: collapse;
}

img {
	 cursor: pointer;
}

/* Second column (details) */
tr:nth-child(even) > td {
	background-color: #ffff95;
}

tr:nth-child(odd) > td {
	background-color: #e1e183;
}

/* First column (labels) */
tr:nth-child(even) > td:first-child {
	vertical-align: top;
	background-color: #ffff85;
}

tr:nth-child(odd) > td:first-child {
	vertical-align: top;
	background-color: #d1d173;
}
