/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * From the ARB_shader_image_load_store spec:
 *
 * "It is an error to declare an image variable where the format
 *  qualifier does not match the image variable type."
 */
layout(rgba32i) uniform uimage2D img;

void main()
{
}
