 /*
  * [config]
  * expect_result: fail
  * glsl_version: 3.0 es
  * require_extensions: GL_EXT_shader_framebuffer_fetch
  * [end config]
  */
#version 300 es
#extension GL_EXT_shader_framebuffer_fetch : enable

inout highp float gl_FragDepth;

void main()
{
    gl_FragDepth += 0.5;
}
