/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * Call a signed integer atomic built-in with a mismatching
 * argument data type.
 */
layout(r32i) uniform iimage1D img;

void main()
{
    imageAtomicCompSwap(img , int(0) , uint(0), uint(0));
}
