/* [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: GL_ARB_gpu_shader_fp64
 * [end config]
 *
 * Check that the following test vectors are constant folded correctly:
 * ceil(dvec4(-2.0lf, -0.6666666666666667lf, 0.6666666666666665lf, 2.0lf)) => dvec4(-2.0lf, -0.0lf, 1.0lf, 2.0lf)
 */
#version 150
#extension GL_ARB_gpu_shader_fp64 : require

void main()
{
  double[distance(ceil(dvec4(-2.0lf, -0.6666666666666667lf, 0.6666666666666665lf, 2.0lf)), dvec4(-2.0lf, -0.0lf, 1.0lf, 2.0lf)) <= 3.0000000000000004e-05lf ? 1 : -1] array0;
  gl_FragColor = vec4(array0.length());
}
