/* This is a generated test, do not edit
 * Generated by gen_extensions_defined.py
 *
 * [config]
 * expect_result: pass
 * glsl_version: 1.10
 * require_extensions: GL_ARB_texture_gather 
 * [end config]
 */

#version 110
#extension GL_ARB_texture_gather : require


#if !defined GL_ARB_texture_gather
#    error GL_ARB_texture_gather is not defined
#elif GL_ARB_texture_gather != 1
#    error GL_ARB_texture_gather != 1
#endif

float foo() { return 0.0; }
