/* This is a generated test, do not edit
 * Generated by gen_extensions_defined.py
 *
 * [config]
 * expect_result: pass
 * glsl_version: 1.40
 * require_extensions: !GL_ARB_fragment_coord_conventions 
 * [end config]
 */

#version 140


#if defined GL_ARB_fragment_coord_conventions
#    error GL_ARB_fragment_coord_conventions is defined, but should not be
#endif

float foo() { return 0.0; }
