/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * From the ARB_shader_image_load_store spec:
 *
 * "Except for array indexing, structure field selection, and
 *  parentheses, images are not allowed to be operands in
 *  expressions."
 */
layout(rgba32f) uniform image2D img;

void main()
{
        img--;
}
