/* This is a generated test, do not edit
 * Generated by gen_extensions_defined.py
 *
 * [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: !GL_ARB_gpu_shader_fp64 
 * [end config]
 */

#version 150


#if defined GL_ARB_gpu_shader_fp64
#    error GL_ARB_gpu_shader_fp64 is defined, but should not be
#endif

float foo() { return 0.0; }
