/* [config]
 * expect_result: pass
 * glsl_version: 4.00
 * require_extensions: GL_ARB_gpu_shader_int64
 * [end config]
 *
 * Check that the following test vectors are constant folded correctly:
 * (i64vec3(167l, 66l, 187l) % i64vec3(167l, 66l, 187l)) => i64vec3(0l, 0l, 0l)
 */
#version 400
#extension GL_ARB_gpu_shader_int64 : require

void main()
{
  float[all(equal((i64vec3(167l, 66l, 187l) % i64vec3(167l, 66l, 187l)), i64vec3(0l, 0l, 0l))) ? 1 : -1] array0;
  gl_Position = vec4(array0.length());
}
