/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * From the ARB_shader_image_load_store spec:
 *
 * "As function parameters, images may only be passed to [arguments]
 *  of matching type."
 */
layout(rgba32f) uniform image2D x;

void f(imageCube y)
{
}

void main()
{
    f(x);
}
