/*
 * [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * Check that the builtin constants defined by the extension
 * are present.
 */
out ivec4 color;

void main()
{
    color.x = gl_MaxImageUnits +
            gl_MaxCombinedImageUnitsAndFragmentOutputs +
            gl_MaxImageSamples +
            gl_MaxVertexImageUniforms +
            gl_MaxGeometryImageUniforms +
            gl_MaxFragmentImageUniforms +
            gl_MaxCombinedImageUniforms;
}
