/*
 * [config]
 * expect_result: fail
 * glsl_version: 3.0 es
 * require_extensions: GL_EXT_shader_framebuffer_fetch_non_coherent
 * [end config]
 */
#version 300 es
#extension GL_EXT_shader_framebuffer_fetch_non_coherent : enable

out highp vec4 color;

void main()
{
    color = gl_LastFragData[0];
}
