/* This is a generated test, do not edit
 * Generated by gen_extensions_defined.py
 *
 * [config]
 * expect_result: pass
 * glsl_version: 1.40
 * require_extensions: GL_ARB_conservative_depth 
 * [end config]
 */

#version 140
#extension GL_ARB_conservative_depth : require


#if !defined GL_ARB_conservative_depth
#    error GL_ARB_conservative_depth is not defined
#elif GL_ARB_conservative_depth != 1
#    error GL_ARB_conservative_depth != 1
#endif

float foo() { return 0.0; }
