/* [config]
 * expect_result: pass
 * glsl_version: 1.30
 * require_extensions: GL_ARB_texture_query_lod
 * [end config]
 */

#version 130
#extension GL_ARB_texture_query_lod : enable

uniform sampler2DArrayShadow s;
varying vec2 coord;

void main()
{
    gl_FragColor.xy = textureQueryLOD(s, coord);
}
