/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.0 es
 * require_extensions: GL_EXT_shader_framebuffer_fetch
 * [end config]
 */
#version 100
#extension GL_EXT_shader_framebuffer_fetch : enable

void main()
{
    gl_LastFragData[0] = vec4(1.0);
}
