/*
 * [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * From the ARB_shader_image_load_store spec:
 *
 * "[Images] can only be declared as function parameters or uniform
 *  variables."
 */
layout(rgba32f) uniform image2DMS img;
layout(rgba32ui) uniform uimage2DMS uimg;
layout(rgba32i) uniform iimage2DMS iimg;

void f(image2DMS arg,
       uimage2DMS uarg,
       iimage2DMS iarg)
{
}

void main()
{
}
