/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.50
 * require_extensions: GL_ARB_shader_image_load_store
 * [end config]
 */
#version 150
#extension GL_ARB_shader_image_load_store: require

/*
 * From the ARB_shader_image_load_store spec:
 *
 * "Fragment shaders also allow the following layout qualifier on 'in'
 *  only (not with variable declarations):
 *
 *  layout-qualifier-id
 *    early_fragment_tests"
 */
layout(early_fragment_tests) uniform b { image2D img; };

void main()
{
}
