/* [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: GL_ARB_gpu_shader_fp64
 * [end config]
 *
 * Check that the following test vectors are constant folded correctly:
 * length(dvec2(-0.1lf, -1.2lf)) => 1.2041594578792296lf
 * length(dvec2(-0.42lf, 0.48lf)) => 0.6378087487640789lf
 */
#version 150
#extension GL_ARB_gpu_shader_fp64 : require

void main()
{
  double[distance(length(dvec2(-0.1lf, -1.2lf)), 1.2041594578792296lf) <= 1.2041594578792297e-05lf ? 1 : -1] array0;
  double[distance(length(dvec2(-0.42lf, 0.48lf)), 0.6378087487640789lf) <= 6.37808748764079e-06lf ? 1 : -1] array1;
  gl_FragColor = vec4(array0.length() + array1.length());
}
