 /*
  * [config]
  * expect_result: fail
  * glsl_version: 1.0 es
  * require_extensions: GL_EXT_shader_framebuffer_fetch
  * [end config]
  */
#version 100
#extension GL_EXT_shader_framebuffer_fetch : enable

inout highp vec4 color;

void main()
{
    color += vec4(0.5);
}
