/* This is a generated test, do not edit
 * Generated by gen_extensinos_defined.py
 *
 * [config]
 * expect_result: fail
 * glsl_version: 1.40
 * require_extensions: !GL_ARB_shader_storage_buffer_object 
 * [end config]
 */

#version 140
#extension GL_ARB_shader_storage_buffer_object : require


float foo() { return 0.0; }
