/* [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: GL_ARB_gpu_shader_fp64
 * [end config]
 *
 * Check that the following test vectors are constant folded correctly:
 * length(dvec3(-1.3333333333333333lf, -0.85lf, -0.94lf)) => 1.8395319452996128lf
 * length(dvec3(1.67lf, 0.66lf, 1.87lf)) => 2.592566296162935lf
 */
#version 150
#extension GL_ARB_gpu_shader_fp64 : require

void main()
{
  double[distance(length(dvec3(-1.3333333333333333lf, -0.85lf, -0.94lf)), 1.8395319452996128lf) <= 1.839531945299613e-05lf ? 1 : -1] array0;
  double[distance(length(dvec3(1.67lf, 0.66lf, 1.87lf)), 2.592566296162935lf) <= 2.5925662961629354e-05lf ? 1 : -1] array1;
  gl_Position = vec4(array0.length() + array1.length());
}
