/*
 * [config]
 * expect_result: fail
 * glsl_version: 1.0 es
 * require_extensions: GL_EXT_shader_framebuffer_fetch
 * [end config]
 */
#version 100
#extension GL_EXT_shader_framebuffer_fetch : enable

layout(noncoherent) mediump vec4 gl_LastFragData[gl_MaxDrawBuffers];

void main()
{
    gl_LastFragData[0];
}
