/* [config]
 * expect_result: pass
 * glsl_version: 1.50
 * require_extensions: GL_ARB_gpu_shader_fp64
 * [end config]
 *
 * Check that the following test vectors are constant folded correctly:
 * normalize(-1.3333333333333333lf) => -1.0lf
 * normalize(0.85lf) => 1.0lf
 */
#version 150
#extension GL_ARB_gpu_shader_fp64 : require

void main()
{
  double[distance(normalize(-1.3333333333333333lf), -1.0lf) <= 1e-05lf ? 1 : -1] array0;
  double[distance(normalize(0.85lf), 1.0lf) <= 1e-05lf ? 1 : -1] array1;
  gl_Position = vec4(array0.length() + array1.length());
}
