/*
 * [config]
 * expect_result: fail
 * glsl_version: 3.0 es
 * require_extensions: GL_EXT_shader_framebuffer_fetch
 * [end config]
 */
#version 300 es
#extension GL_EXT_shader_framebuffer_fetch : enable

layout(noncoherent) inout mediump vec4 fcolor;

void main()
{
    fcolor;
}
